Ext.define('DSS.component.TimeSpanControl', {
	extend : 'DSS.component.Base',
	mixins : {
		observable : 'Ext.util.Observable'
	},
	numHour : null,
	numMinutes : null,
	numSeconds : null,
	timeField : null,
	padding : 5,
	lblHMS : null,
	lblFieldLabel : null,
	fHideHour : false,
	fHideMinute : false,
	fHideSecond : false,
	fWithEnabler : false,
	minValue:null,
	maxValue:null,
	setMinValue : function(minValue) {
		this.minValue=minValue;
		if (minValue==null)
			return;
		if (this.getValue()<minValue)
			this.setValue(minValue);
	},
	setMaxValue : function (maxValue) {
		this.maxValue=maxValue;
		if (maxValue==null)
			return;
		if (this.getValue()>maxValue)
			this.setValue(maxValue);
	},
	updateToggle : function() {
		var fToggled = this.lblFieldLabel.pressed;
		this.numHour.setDisabled(!fToggled);
		this.numMinutes.setDisabled(!fToggled);
		this.numSeconds.setDisabled(!fToggled);
		if (fToggled)
			this.fireEvent('enabledChange');
		else
			this.fireEvent('disabledChange');
	},
	constructor : function(config) {
		var mThis = this;
		this.fHideHour = false;
		this.fHideMinute = false;
		this.fHideSecond = false;
		this.fWithEnabler = false;
		if (config.hideHour != null)
			this.fHideHour = config.hideHour;
		if (config.hideMinute != null)
			this.fHideMinute = config.hideMinute;
		if (config.hideSecond != null)
			this.fHideSecond = config.hideSecond;
		if (config.withEnabler != null)
			this.fWithEnabler = config.withEnabler;
		this.numHour = Ext.create('Ext.form.field.Number', {
			allowBlank : false,
			blankText : _('This field is required'),
			invalidText : _('The value in this field is invalid'),
			allowDecimals : false,
			margin : ((!this.fHideMinute) || (!this.fHideSecond)) ? '0 3 0 0' : '0 0 0 0',
			width : 42,
			value : 0,
			minValue : 0,
			maxValue : 23,
			minText : _("The minimum value for hour is 0"),
			maxText : _("The maximum value for hour is 23"),
			listeners : {
				'change' : function(oThis,newValue,oldValue) {
					if (mThis.minValue!=null)
						if (mThis.getValue()<mThis.minValue) {
							oThis.setValue(oldValue);
							return;
						}
					if (mThis.maxValue!=null)
						if (mThis.getValue()>mThis.maxValue) {
							oThis.setValue(oldValue);
							return;
						}
					
					mThis.fireEvent('change');
				}
			}

		});
		this.numMinutes = Ext.create('Ext.form.field.Number', {
			allowBlank : false,
			blankText : _('This field is required'),
			invalidText : _('The value in this field is invalid'),
			allowDecimals : false,
			margin : (!this.fHideSecond) ? '0 3 0 0' : '0 0 0 0',
			width : 42,
			value : 0,
			minValue : 0,
			maxValue : 59,
			minText : _("The minimum value for minute is 0"),
			maxText : _("The maximum value for minute is 59"),
			listeners : {
				'change' : function(oThis,newValue,oldValue) {
					if (mThis.minValue!=null)
						if (mThis.getValue()<mThis.minValue) {
							oThis.setValue(oldValue);
							return;
						}
					if (mThis.maxValue!=null)
						if (mThis.getValue()>mThis.maxValue) {
							oThis.setValue(oldValue);
							return;
						}
					mThis.fireEvent('change');
				}
			}

		});
		this.numSeconds = Ext.create('Ext.form.field.Number', {
			allowBlank : false,
			blankText : _('This field is required'),
			invalidText : _('The value in this field is invalid'),
			allowDecimals : false,
			width : 42,
			value : 0,
			minValue : 0,
			maxValue : 59,
			minText : _("The minimum value for second is 0"),
			maxText : _("The maximum value for second is 59"),
			listeners : {
				'change' : function(oThis,newValue,oldValue) {
					if (mThis.minValue!=null)
						if (mThis.getValue()<mThis.minValue) {
							oThis.setValue(oldValue);
							return;
						}
					if (mThis.maxValue!=null)
						if (mThis.getValue()>mThis.maxValue) {
							oThis.setValue(oldValue);
							return;
						}
					mThis.fireEvent('change');
				}
			}

		});
		if (this.fWithEnabler) {
			this.lblFieldLabel = Ext.create('Ext.button.Button', {
				text : config.fieldLabel,
				width : config.labelWidth,
				enableToggle : true,
				margin : '0 5 0 0',
				listeners : {
					toggle : function() {
						mThis.updateToggle();
					}
				}
			});
			this.lblFieldLabel.toggle(true);
			this.updateToggle();
		} else {
			this.lblFieldLabel = Ext.create('Ext.form.Label', {
				html : '<span style="color:#005138">' + config.fieldLabel + ": </span>",
				align : 'middle',
				margin : '2 0 0 0',
				width : config.labelWidth
			});
		}

		var sLabel = "";

		if (!this.fHideHour)
			sLabel = "hh";
		if (!this.fHideMinute) {
			if (sLabel.length > 0)
				sLabel += ":";
			sLabel += "mm";
		}
		if (!this.fHideSecond) {
			if (sLabel.length > 0)
				sLabel += ":";
			sLabel += "ss";
		}

		this.lblHMS = Ext.create('Ext.form.Label', {
			text : '(' + _(sLabel) + ')',
			align : 'middle',
			margin : '2 0 0 5',
			style : {
				color : '#005138'
			}
		});
		var oItems = [ this.lblFieldLabel ];

		if (!this.fHideHour)
			oItems.push(this.numHour);
		if (!this.fHideMinute)
			oItems.push(this.numMinutes);
		if (!this.fHideSecond)
			oItems.push(this.numSeconds);

		oItems.push(this.lblHMS);
		if (config.labelWidth == null)
			config.labelWidth = 100;
		/*
		 * this.timeField = Ext.create('Ext.form.FieldContainer', { fieldLabel :
		 * config.fieldLabel, labelWidth : config.labelWidth, border : 1, items :
		 * oItems, layout : 'column', width : (700 - config.labelWidth) });
		 */
		this.timeField = Ext.create('Ext.panel.Panel', {
			border : false,
			layout : 'column',
			items : oItems,
			width : 700
		})

		Ext.apply(this, {
			items : [ this.timeField ],
			padding : "0 0 0 0"
		});
		this.callParent(arguments);
	},
	reset : function() {
		this.numHour.setRawValue(null);
		this.numMinutes.setRawValue(null);
		this.numSeconds.setRawValue(null);
		if (this.fWithEnabler) {
			this.lblFieldLabel.toggle(false);
			this.updateToggle();
		}
	},
	setValue : function(value) {
		if (this.fWithEnabler) {
			if (value == null) {
				this.reset();
			} else {
				this.lblFieldLabel.toggle(true);
				this.updateToggle();
			}
		}
		var iSeconds = value % 60;
		var iMinutes = Math.floor(value / 60) % 60;
		var iHour = Math.floor((value / 60) / 60);
		if (!this.fHideHour)
			this.numHour.setValue(iHour);
		if (!this.fHideMinute)
			this.numMinutes.setValue(iMinutes);
		if (!this.fHideSecond)
			this.numSeconds.setValue(iSeconds);
	},
	isValid : function() {
		var fValue = true;
		if (this.fWithEnabler) {
			if (!this.lblFieldLabel.pressed)
				return true;
		}		
		if (!this.fHideHour)
			fValue = fValue && this.numHour.isValid();
		if (!this.fHideMinute)
			fValue = fValue && this.numMinutes.isValid();
		if (!this.fHideSecond)
			fValue = fValue && this.numSeconds.isValid();
		return fValue;
	},
	getValue : function() {
		if (this.fWithEnabler) {
			if (!this.lblFieldLabel.pressed)
				return null;
		}		
		var iReturnValue = 0;
		if (!this.fHideHour)
			iReturnValue += (this.numHour.getValue() * 60) * 60;
		if (!this.fHideMinute)
			iReturnValue += this.numMinutes.getValue() * 60;
		if (!this.fHideSecond)
			iReturnValue += this.numSeconds.getValue();
		return iReturnValue;
	}
});